package TextDigest;

import java.io.*;
import com.Ostermiller.Syntax.Lexer.*;
import Main.TextManager;
import java.util.HashSet;


/**
 * Lexer czytajacy kod IL i rozpoznajacy jego gramatyke.
 * Wczytujac slowo dopasowuje je do stanu w jakim sie znajduje oraz wyznacza
 * stan do jakiego przechodzi. Generuje BLToken'y zawierajace informacje
 * o stanie w ktorym zostaly wczytane. Na podstawie tych stanow mozna dobrac
 * kolory do fragmentow tekstu, poprawnie ustawic stan lexera podczas wznawiania
 * kolorowania w srodku dokumentu, jak rowniez przerwac kolorowanie, gdy
 * interpretacja dalszej czesci dokumentu sie nie zmienila.
 *
 */

%%


%public
%class BLLexer
%implements Lexer
%function getNextToken
%type Token 



%init{ /* dodatkowe instrukcje konstruktora */
    keywrds = new HashSet();
    keywrds.add("int");
    keywrds.add("bool");
    keywrds.add("buffer");
    keywrds.add("define");
    keywrds.add("delay");
    keywrds.add("false");
    keywrds.add("from");
    keywrds.add("get");
    keywrds.add("if");
    keywrds.add("in");
    keywrds.add("infty");
    keywrds.add("int");
    keywrds.add("length");
    keywrds.add("main");
    keywrds.add("process");
    keywrds.add("prop");
    keywrds.add("put");
    keywrds.add("size");
    keywrds.add("stateset");
    keywrds.add("to");
    keywrds.add("true");
    keywrds.add("urgent");
    keywrds.add("var");
    keywrds.add("when");
%init}


%eofval{
	inbexpr = false;
	canget = false;
	return null;
%eofval}


%{  /* dodatkowe deklaracje wewnatrz klasy */
    private int lastToken;
    private int nextState=YYINITIAL;
    
    private int WHERE_DOC;
    
    private int WHERE_BEXPR;
    private int bexp_pcount;
    public boolean canget = false;
    private boolean inbexpr = false;
    private boolean bexpnum;
    private boolean bexprel;
    private boolean bexpbop;
    private boolean bexpboo;
    private HashSet keywrds;
    private TextManager tman = null;
    
    /** 
     * next Token method that allows you to control if whitespace and comments are
     * returned as tokens.
     */
    public Token getNextToken(boolean returnComments, boolean returnWhiteSpace)throws IOException{
        Token t = getNextToken();
        while (t != null && ((!returnWhiteSpace && t.isWhiteSpace()) || (!returnComments && t.isComment()))){
            t = getNextToken();
        }
        return (t); 
    }
    
    public static boolean goodForStop(int state){
        return state >= CONST_DECL && state <= TRANS_INSTR;
    }

    public void setState(int st){
    	yybegin(st);
    }

    public int getState(){
    	return zzLexicalState;
    }

    
    public void setTMan(TextManager tm){
    	tman = tm;
    }
    
    
    
    public void startBexpr(int state){
	if(inbexpr) {
		lastToken = BLToken.ERROR;
		System.out.println("tell developers of GraphIL, that you saw this message. (-1):"+state);
	}
    	WHERE_BEXPR = state;
    	bexp_pcount = 0;
    	inbexpr = true;
    	bexpnum = false;
    	bexprel = false;
    	bexpbop = false;
    	bexpboo = false;
	yybegin(BEXPR);
    }
    

        
    /**
     * Metoda ta sprawdza, czy jakie slowo kluczowe nie jest uzywane jako identyfikator.
     * @return odpowiedni numer tokena (IDENTIFIER jesli ok, RW_WRONG_PLACE, jesli slowo kluczowe).
     */
    private int checkIdentifier(String ident){
    	if(keywrds.contains(ident))
        	return BLToken.RW_WRONG_PLACE;
    	return BLToken.IDENTIFIER;
    }
    
    private void bexpDebug(){
    	System.out.println();
        System.out.print("bdb: inbexpr: "+inbexpr+" bexpnum: "+bexpnum+" bexprel: "+bexprel+" bexpbop: "
        		+bexpbop+" bexpboo: "+bexpboo+" count: "+bexp_pcount);
    }
    
    /**
     * Closes the current input stream, and resets the scanner to read from a new input stream.
     * All internal variables are reset, the old input stream  cannot be reused
     * (content of the internal buffer is discarded and lost).
     * The lexical state is set to the initial state.
     * Subsequent tokens read from the lexer will start with the line, char, and column
     * values given here.
     *
     * @param reader The new input.
     * @param yyline The line number of the first token.
     * @param yychar The position (relative to the start of the stream) of the first token.
     * @param yycolumn The position (relative to the line) of the first token.
     * @throws IOException if an IOExecption occurs while switching readers.
     */
    public void reset(java.io.Reader reader, int yyline, int yychar, int yycolumn) throws IOException{
        yyreset(reader);
        this.yyline = yyline;
		this.yychar = yychar;
		this.yycolumn = yycolumn;
	}
%}


%line
%char
%full

BLANK=([ ])
TAB=([\t])
FF=([\f])
CR=([\r])
LF=([\n])
EOL=({CR}|{LF}|{CR}{LF})
WhiteSpace=({BLANK}|{TAB}|{FF}|{EOL})
AnyNonSeparator=([^\t\f\r\n\ \(\)\{\}\[\]\;\,\.\=\>\<\!\~\:\+\-\*\/\%\|\&])
Digit=([0-9])
Number=({Digit}+)
BLLetter=([a-zA-Z_])

Identifier=({BLLetter}({BLLetter}|{Digit})*)
ProgramId=({Identifier})
ConstId=({Identifier})
ProcessId=({Identifier})
VarId=({Identifier})
BufId=({Identifier})
StateId=({Identifier})
PropId=({Identifier})
ErrorIdentifier=({AnyNonSeparator}+)


TradCommentBegin=("/*")
DocCommentBegin=("/**")
NonTermStars=([^\*\/]*[\*]+[^\*\/])
TermStars=([\*]+[\/])
CommentText=((([^\*]*[\/])|{NonTermStars})*)
CommentEnd=([^\*]*{TermStars})
TradComment=({TradCommentBegin}{CommentText}{CommentEnd})
DocCommentEnd1=([^\/\*]{CommentText}{CommentEnd})
DocCommentEnd2=({NonTermStars}{CommentText}{CommentEnd})
DocComment=({DocCommentBegin}({DocCommentEnd1}|{DocCommentEnd2}|{TermStars}|[\/]))
OpenComment=({TradCommentBegin}{CommentText}([^\*]*)([\*]*))

Type=("int"|"bool")
LogConst=("true"|"false")
Plus=("+")
Minus=("-")
Mult=("*")
Divi=("/")
Mod=("%")
OP=({Plus}|{Minus}|{Divi}|{Mod}|{Mult})
BOP=("||"|"&&")
RELOP=("<"|">"|"<="|">="|"=="|"!=")

Ldelay=("["|"(")
Rdelay=("]"|")")
Bound=({Number}|"infty"|{ConstId})

Anything=([^a]|[a])



/**************************************************************/
/*  DLA EFEKTYWNOSCI goodForStop STANY "POCZATKOWE" SA RAZEM  */
/*  MAM NADZIEJE, ZE JFLEX ZAWSZE PRZYDZIELA KOLEJNE NUMERY   */
/*                                                            */
/*         (mógłby chcieć potem optymalizować...)             */
/**************************************************************/

%state CONST_DECL
%state VAR_DECL
%state BUF_DECL
%state PROC_DEF
%state LOCVAR_DECL
%state TRANS_FROM
%state PROP
%state INIT_INSTR
%state TRANS_INSTR



%state PROGRAM

%state CONST_NAME
%state CONST_VALUE
%state CONST_END

%state VAR_TYPE
%state VAR_NAME
%state VAR_ASSIGN
%state VAR_VALUE
%state VAR_END

%state BUF_TYPE
%state BUF_NAME
%state BUF_SIZE
%state BUF_ASSIGN
%state BUF_VALUE
%state BUF_END

%state PROC_NAME
%state PROC_LP
%state PROC_TYPE
%state PROC_PARAM
%state PROC_RP
%state PROC

%state STATESET
%state STATESET_NAME
%state STATESET_END

%state LOCVAR_TYPE
%state LOCVAR_NAME
%state LOCVAR_ASSIGN
%state LOCVAR_VALUE
%state LOCVAR_END

%state INIT
%state INIT_NAME
%state INIT_COLON
%state INIT_INSTR_ASSIGN
%state INIT_BEXPR
%state INIT_BEXPR_END
%state INIT_END

%state TRANS_NAME1
%state TRANS_TO
%state TRANS_NAME2
%state TRANS_COLON
%state TRANS_WHEN
%state WHEN
%state WHEN_END
%state TRANS_DELAY
%state DELAY_LPAR
%state DELAY_BOUND1
%state DELAY_COMMA
%state DELAY_BOUND2
%state DELAY_RPAR
%state DELAY_END
%state URGENT
%state TRANS

%state TRANS_INSTR_ASSIGN
%state TRANS_BEXPR_END
%state PUT_LPAR
%state PUT_RPAR
%state PUT_NAME
%state PUT_COMMA
%state PUT_END

%state GET_LPAR
%state GET_BUF
%state GET_COMMA
%state GET_VAR
%state GET_RPAR

%state MAIN
%state MAIN_NAME
%state MAIN_LP
%state MAIN_RP
%state MAIN_NUM
%state MAIN_END

%state PROP_L
%state PROP_ID
%state PROP_COLON
%state PROP_IN
%state PROP_SNAME
%state PROP_END
%state PROC_END

%state MPROP
%state MPROP_LB
%state MPROP_PROCESS
%state MPROP_LP
%state MPROP_NUM
%state MPROP_RP
%state MPROP_DOT
%state MPROP_LPROP
%state MPROP_END
%state END

%state BEXPR
%state BEXPR_MIDDLE
%state LEN_LPAR
%state LEN_RPAR
%state LEN_BUF

%state BLDOC
%state COMMENT

%state END
%state ANYTHING

%%

<YYINITIAL> {ProgramId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    if(tman != null)
    	tman.checkName(text);
    yybegin(PROGRAM);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROGRAM> "{" {
    lastToken = BLToken.SEPARATOR_LBRACE;
    String text = yytext();
    yybegin(CONST_DECL);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<CONST_DECL> "define" {
    lastToken = BLToken.RESERVED_WORD_DEFINE;
    String text = yytext();
    yybegin(CONST_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<CONST_NAME> {ConstId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(CONST_VALUE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<CONST_VALUE> {Number} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(CONST_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<CONST_END> ";" {
    lastToken = BLToken.SEPARATOR_SEMICOLON;
    String text = yytext();
    yybegin(CONST_DECL);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<VAR_DECL, CONST_DECL> "var" {
    lastToken = BLToken.RESERVED_WORD_VAR;
    String text = yytext();
    yybegin(VAR_TYPE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<VAR_TYPE> {Type} {
    lastToken = BLToken.TYPE;
    String text = yytext();
    yybegin(VAR_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<VAR_NAME> {VarId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(VAR_ASSIGN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<VAR_ASSIGN> "=" {
    lastToken = BLToken.OPERATOR_ASSIGN;
    String text = yytext();
    yybegin(VAR_VALUE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<VAR_VALUE> {Number} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(VAR_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<VAR_VALUE> {LogConst} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(VAR_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<VAR_ASSIGN, VAR_END> "," {
    lastToken = BLToken.SEPARATOR_COMMA;
    String text = yytext();
    yybegin(VAR_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<VAR_ASSIGN, VAR_END> ";" {
    lastToken = BLToken.SEPARATOR_SEMICOLON;
    String text = yytext();
    yybegin(VAR_DECL);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<BUF_DECL, VAR_DECL, CONST_DECL> "buffer" {
    lastToken = BLToken.RESERVED_WORD_BUFFER;
    String text = yytext();
    yybegin(BUF_TYPE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<BUF_TYPE> {Type} {
    lastToken = BLToken.TYPE;
    String text = yytext();
    yybegin(BUF_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<BUF_NAME> {BufId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(BUF_SIZE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<BUF_SIZE> "size" {
    lastToken = BLToken.RESERVED_WORD_SIZE;
    String text = yytext();
    yybegin(BUF_ASSIGN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<BUF_ASSIGN> "=" {
    lastToken = BLToken.OPERATOR_ASSIGN;
    String text = yytext();
    yybegin(BUF_VALUE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<BUF_VALUE> {Number} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(BUF_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<BUF_SIZE, BUF_END> "," {
    lastToken = BLToken.SEPARATOR_COMMA;
    String text = yytext();
    yybegin(BUF_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<BUF_SIZE, BUF_END> ";" {
    lastToken = BLToken.SEPARATOR_SEMICOLON;
    String text = yytext();
    yybegin(BUF_DECL);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<PROC_DEF, BUF_DECL, VAR_DECL, CONST_DECL> "process" {
    lastToken = BLToken.RESERVED_WORD_PROCESS;
    String text = yytext();
    yybegin(PROC_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROC_NAME> {ProcessId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(PROC_LP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROC_LP> "(" {
    lastToken = BLToken.SEPARATOR_LPAREN;
    String text = yytext();
    yybegin(PROC_TYPE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROC_TYPE> {Type} {
    lastToken = BLToken.TYPE;
    String text = yytext();
    yybegin(PROC_PARAM);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROC_PARAM> {VarId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(PROC_RP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROC_RP> ")" {
    lastToken = BLToken.SEPARATOR_RPAREN;
    String text = yytext();
    yybegin(PROC);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROC, PROC_LP> "{" {
    lastToken = BLToken.SEPARATOR_LBRACE;
    String text = yytext();
    yybegin(STATESET);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<STATESET> "stateset" {
    lastToken = BLToken.RESERVED_WORD_STATESET;
    String text = yytext();
    yybegin(STATESET_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<STATESET_NAME> {StateId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(STATESET_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<STATESET_END> "," {
    lastToken = BLToken.SEPARATOR_COMMA;
    String text = yytext();
    yybegin(STATESET_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<STATESET_END> ";" {
    lastToken = BLToken.SEPARATOR_SEMICOLON;
    String text = yytext();
    yybegin(LOCVAR_DECL);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<LOCVAR_DECL> "var" {
    lastToken = BLToken.RESERVED_WORD_VAR;
    String text = yytext();
    yybegin(LOCVAR_TYPE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LOCVAR_TYPE> {Type} {
    lastToken = BLToken.TYPE;
    String text = yytext();
    yybegin(LOCVAR_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LOCVAR_NAME> {VarId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(LOCVAR_ASSIGN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LOCVAR_ASSIGN> "=" {
    lastToken = BLToken.OPERATOR_ASSIGN;
    String text = yytext();
    yybegin(LOCVAR_VALUE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LOCVAR_VALUE> {Number} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(LOCVAR_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LOCVAR_ASSIGN, LOCVAR_END> "," {
    lastToken = BLToken.SEPARATOR_COMMA;
    String text = yytext();
    yybegin(LOCVAR_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LOCVAR_ASSIGN, LOCVAR_END> ";" {
    lastToken = BLToken.SEPARATOR_SEMICOLON;
    String text = yytext();
    yybegin(LOCVAR_DECL);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<LOCVAR_DECL> "to" {
    lastToken = BLToken.RESERVED_WORD_TO;
    String text = yytext();
    yybegin(INIT_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<INIT_NAME> {StateId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(INIT_COLON);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<INIT_COLON> ":" {
    lastToken = BLToken.OPERATOR_COLON;
    String text = yytext();
    yybegin(INIT);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<INIT> "{" {
    lastToken = BLToken.SEPARATOR_LBRACE;
    String text = yytext();
    yybegin(INIT_INSTR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<INIT_INSTR> {VarId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(INIT_INSTR_ASSIGN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<INIT_INSTR_ASSIGN> "=" {
    if(inbexpr) {
	lastToken = BLToken.ERROR;
	System.out.println("tell developers of GraphIL, that you saw this message. (1)");
    } else
	lastToken = BLToken.OPERATOR_ASSIGN;
    WHERE_BEXPR = INIT_BEXPR_END;
    bexp_pcount = 0;
    inbexpr = true;
    bexpnum = false;
    bexprel = false;
    bexpbop = false;
    bexpboo = false;
    String text = yytext();
    yybegin(BEXPR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<INIT_BEXPR_END> ";" {
    if((bexprel && bexpboo && !bexpbop)
    || (bexpbop && bexpnum && !bexprel))
	lastToken = BLToken.ERROR;
    else {
	lastToken = BLToken.SEPARATOR_SEMICOLON;
	yybegin(INIT_INSTR);
        inbexpr = false;
    }
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<INIT_INSTR> "}" {
    lastToken = BLToken.SEPARATOR_RBRACE;
    String text = yytext();
    yybegin(INIT_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<TRANS_FROM, INIT_END> "from" {
    lastToken = BLToken.RESERVED_WORD_FROM;
    String text = yytext();
    yybegin(TRANS_NAME1);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<TRANS_NAME1> {StateId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(TRANS_TO);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<TRANS_TO> "to" {
    lastToken = BLToken.RESERVED_WORD_TO;
    String text = yytext();
    yybegin(TRANS_NAME2);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<TRANS_NAME2> {StateId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(TRANS_COLON);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<TRANS_COLON> ":" {
    lastToken = BLToken.OPERATOR_COLON;
    String text = yytext();
    yybegin(TRANS_WHEN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<TRANS_WHEN> "when" {
    lastToken = BLToken.RESERVED_WORD_WHEN;
    String text = yytext();
    yybegin(WHEN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<WHEN> "(" {
    lastToken = BLToken.SEPARATOR_LPAREN;
    String text = yytext();
    if(inbexpr)
	System.out.println("tell developers of GraphIL, that you saw this message. (2)");
    WHERE_BEXPR = WHEN_END;
    bexp_pcount = 0;
    canget = true;
    inbexpr = true;
    bexpnum = false;
    bexprel = false;
    bexpbop = false;
    bexpboo = false;
    yybegin(BEXPR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<WHEN_END> ")" {
    lastToken = BLToken.SEPARATOR_RPAREN;
    String text = yytext();
    inbexpr = false;
    canget = false;
    yybegin(TRANS_DELAY);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<TRANS_DELAY, TRANS_WHEN> "delay" {
    lastToken = BLToken.RESERVED_WORD_DELAY;
    String text = yytext();
    yybegin(DELAY_LPAR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<DELAY_LPAR> {Ldelay} {
    lastToken = BLToken.SEPARATOR_LPAREN;
    String text = yytext();
    yybegin(DELAY_BOUND1);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<DELAY_BOUND1> {Bound} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(DELAY_COMMA);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<DELAY_COMMA> "," {
    lastToken = BLToken.SEPARATOR_COMMA;
    String text = yytext();
    yybegin(DELAY_BOUND2);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<DELAY_BOUND2> {Bound} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(DELAY_RPAR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<DELAY_RPAR> {Rdelay} {
    lastToken = BLToken.SEPARATOR_RPAREN;
    String text = yytext();
    yybegin(DELAY_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<DELAY_END> ":" {
    lastToken = BLToken.OPERATOR_COLON;
    String text = yytext();
    yybegin(URGENT);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<URGENT> "urgent" {
    lastToken = BLToken.RESERVED_WORD_URGENT;
    String text = yytext();
    yybegin(TRANS);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<TRANS, TRANS_WHEN, TRANS_DELAY, DELAY_END> "{" {
    lastToken = BLToken.SEPARATOR_LBRACE;
    String text = yytext();
    yybegin(TRANS_INSTR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
    
/*************************************************
 *  ten drugi stan pojawil sie tu dlatego,       *
 *  ze ma byc zgoda na procesy bez tranzycji     *
 *************************************************/
<TRANS_FROM, INIT_END> "}" {
    lastToken = BLToken.SEPARATOR_RBRACE;
    String text = yytext();
    yybegin(PROC_DEF);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<TRANS_INSTR> "put" {
    lastToken = BLToken.RESERVED_WORD_PUT;
    String text = yytext();
    yybegin(PUT_LPAR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PUT_LPAR> "(" {
    lastToken = BLToken.SEPARATOR_LPAREN;
    String text = yytext();
    yybegin(PUT_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PUT_NAME> {BufId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(PUT_COMMA);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PUT_COMMA> "," {
    lastToken = BLToken.SEPARATOR_COMMA;
    startBexpr(PUT_RPAR);
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PUT_RPAR> ")" {
    if((bexprel && bexpboo && !bexpbop)
    || (bexpbop && bexpnum && !bexprel))
	lastToken = BLToken.ERROR;
    else {
	lastToken = BLToken.SEPARATOR_RPAREN;
	yybegin(PUT_END);
        inbexpr = false;
    }
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PUT_END> ";" {
    lastToken = BLToken.SEPARATOR_COMMA;
    String text = yytext();
    yybegin(TRANS_INSTR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<TRANS_INSTR> {VarId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(TRANS_INSTR_ASSIGN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<TRANS_INSTR_ASSIGN> "=" {
    if(inbexpr) {
	lastToken = BLToken.ERROR;
	System.out.println("tell developers of GraphIL, that you saw this message. (3)");
    } else
	lastToken = BLToken.OPERATOR_ASSIGN;
    WHERE_BEXPR = TRANS_BEXPR_END;
    bexp_pcount = 0;
    inbexpr = true;
    bexpnum = false;
    bexprel = false;
    bexpbop = false;
    bexpboo = false;
    String text = yytext();
    yybegin(BEXPR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<TRANS_BEXPR_END> ";" {
    lastToken = BLToken.SEPARATOR_SEMICOLON;
    inbexpr = false;
    String text = yytext();
    yybegin(TRANS_INSTR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<TRANS_INSTR> "}" {
    lastToken = BLToken.SEPARATOR_RBRACE;
    String text = yytext();
    yybegin(TRANS_FROM);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

/*************************************************
 *  ten drugi stan pojawil sie tu dlatego,       *
 *  ze ma byc zgoda na procesy bez tranzycji     *
 *************************************************/
<TRANS_FROM, INIT_END> "prop" {
    lastToken = BLToken.RESERVED_WORD_PROP;
    String text = yytext();
    yybegin(PROP_L);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_L> "{" {
    lastToken = BLToken.SEPARATOR_LBRACE;
    String text = yytext();
    yybegin(PROP_ID);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_ID> {PropId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(PROP_COLON);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_COLON> ":" {
    if(inbexpr) {
	lastToken = BLToken.ERROR;
	System.out.println("tell developers of GraphIL, that you saw this message. (4)");
    } else
	lastToken = BLToken.OPERATOR_COLON;
    String text = yytext();
    WHERE_BEXPR = PROP_END;
    bexp_pcount = 0;
    inbexpr = true;
    bexpnum = false;
    bexprel = false;
    bexpbop = false;
    yybegin(PROP_IN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
/* hack: stan PROP_IN musi byc jeszcze wpisany mniejwiecej wszedzie tam gdzie BEXP */
<PROP_IN> "in" {
    inbexpr = false;
    lastToken = BLToken.RESERVED_WORD_IN;
    String text = yytext();
    yybegin(PROP_SNAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<PROP_SNAME> {StateId} {
    inbexpr = false;
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(PROP_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_END> ";" {
    inbexpr = false;
    lastToken = BLToken.SEPARATOR_SEMICOLON;
    String text = yytext();
    yybegin(PROP_ID);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_ID> "}" {
    lastToken = BLToken.SEPARATOR_RBRACKET;
    String text = yytext();
    yybegin(PROC_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROC_END> "}" {
    lastToken = BLToken.SEPARATOR_RBRACKET;
    String text = yytext();
    yybegin(PROC_DEF);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<PROC_DEF> "main" {
    lastToken = BLToken.RESERVED_WORD_MAIN;
    String text = yytext();
    yybegin(MAIN);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MAIN> "{" {
    lastToken = BLToken.SEPARATOR_LBRACE;
    String text = yytext();
    yybegin(MAIN_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MAIN_NAME, MAIN_END> {ProcessId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(MAIN_LP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MAIN_LP> "(" {
    lastToken = BLToken.SEPARATOR_LPAREN;
    String text = yytext();
    yybegin(MAIN_NUM);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MAIN_NUM> {Number} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(MAIN_RP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MAIN_NUM> {LogConst} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(MAIN_RP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MAIN_RP> ")" {
    lastToken = BLToken.SEPARATOR_RPAREN;
    String text = yytext();
    yybegin(MAIN_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MAIN_END, MAIN_LP> "||" {
    lastToken = BLToken.OPERATOR_LOGICAL_OR;
    String text = yytext();
    yybegin(MAIN_NAME);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MAIN_END, MAIN_LP> "}" {
    lastToken = BLToken.SEPARATOR_RBRACE;
    String text = yytext();
    yybegin(MPROP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}


<MPROP> "prop" {
    lastToken = BLToken.RESERVED_WORD_PROP;
    String text = yytext();
    yybegin(MPROP_LB);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MPROP_LB> "{" {
    lastToken = BLToken.SEPARATOR_LBRACKET;
    String text = yytext();
    yybegin(MPROP_PROCESS);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MPROP_PROCESS> {ProcessId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(MPROP_LP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<MPROP_LP> "(" {
    lastToken = BLToken.SEPARATOR_LPAREN;
    String text = yytext();
    yybegin(MPROP_NUM);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MPROP_NUM> {Number} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(MPROP_RP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MPROP_NUM> {LogConst} {
    lastToken = BLToken.NUMBER;
    String text = yytext();
    yybegin(MPROP_RP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MPROP_RP> ")" {
    lastToken = BLToken.SEPARATOR_RPAREN;
    String text = yytext();
    yybegin(MPROP_DOT);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MPROP_DOT, MPROP_LP> "." {
    lastToken = BLToken.OPERATOR;
    String text = yytext();
    yybegin(MPROP_LPROP);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MPROP_LPROP> {PropId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(MPROP_END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<MPROP_END> ";" {
    lastToken = BLToken.SEPARATOR_SEMICOLON;
    String text = yytext();
    inbexpr = false;
    yybegin(MPROP_PROCESS);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<MPROP_LP> ":" {
    lastToken = BLToken.OPERATOR_COLON;
    String text = yytext();
    WHERE_BEXPR = MPROP_END;
    bexp_pcount = 0;
    inbexpr = true;
    bexpnum = false;
    bexprel = false;
    bexpbop = false;
    yybegin(BEXPR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<MPROP_PROCESS> "}" {
    lastToken = BLToken.SEPARATOR_RBRACKET;
    String text = yytext();
    yybegin(END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<END, MPROP> "}" {
    lastToken = BLToken.SEPARATOR_RBRACE;
    String text = yytext();
    yybegin(END);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}







/***************************************************************************************************************/
/*********  aby miec swoje BEXPRESSION, dopisz: ****************************************************************/
/************ swoja wersje zakonczenia ("po WHERE_BEXPR") /inbexpr = false;/ ***********************************/
/************ swoj stan "WHERE_BEXPR" przed BEXPR_MIDDLE {OP} **************************************************/
/************ jak zaczynasz BEXPR, to sprawdz i ustaw czy nie inbexpr (a tez bexpnum, bexprel, bexpbop) ********/
/***************************************************************************************************************/
/*****************   BEDZIEMY AKCEPTOWAC WYRAZENIA ARYCZNE I LORYTMETYCZNE   ***********************************/
/***************************************************************************************************************/

<PROP_IN, BEXPR> "length" {
    lastToken = BLToken.RESERVED_WORD_LENGTH;
    String text = yytext();
    bexpnum = true;
    yybegin(LEN_LPAR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LEN_LPAR> "(" {
    lastToken = BLToken.SEPARATOR_LPAREN;
    String text = yytext();
    yybegin(LEN_BUF);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LEN_BUF> {BufId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(LEN_RPAR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<LEN_RPAR> ")" {
    lastToken = BLToken.SEPARATOR_RPAREN;
    String text = yytext();
    if(bexp_pcount == 0)
	yybegin(WHERE_BEXPR);
    else
	yybegin(BEXPR_MIDDLE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<BEXPR> "get" {
    if(canget){
      lastToken = BLToken.RESERVED_WORD_GET;
      bexpboo = true;
      yybegin(GET_LPAR);
    } else {
      lastToken = BLToken.RW_WRONG_PLACE;
    }
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<GET_LPAR> "(" {
    lastToken = BLToken.SEPARATOR_RPAREN;
    String text = yytext();
    yybegin(GET_BUF);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<GET_BUF> {BufId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(GET_COMMA);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<GET_COMMA> "," {
    lastToken = BLToken.SEPARATOR_COMMA;
    String text = yytext();
    yybegin(GET_VAR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<GET_VAR> {VarId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    yybegin(GET_RPAR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<GET_RPAR> ")" {
    lastToken = BLToken.SEPARATOR_RPAREN;
    String text = yytext();
    if(bexp_pcount == 0)
	yybegin(WHERE_BEXPR);
    else
	yybegin(BEXPR_MIDDLE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<PROP_IN, BEXPR> {LogConst} {
    if(bexpnum)
    	lastToken = BLToken.ERROR;
    else {
	lastToken = BLToken.NUMBER;
	if(bexp_pcount == 0)
	    yybegin(WHERE_BEXPR);
    	else
	    yybegin(BEXPR_MIDDLE);
	bexpboo = true;
	bexpnum = false;
    }
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_IN, BEXPR> {VarId} {
    String text = yytext();
    lastToken = checkIdentifier(text);
    if(bexp_pcount == 0)
	yybegin(WHERE_BEXPR);
    else
	yybegin(BEXPR_MIDDLE);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_IN, BEXPR> "(" {
    lastToken = BLToken.SEPARATOR_LPAREN;
    String text = yytext();
    bexp_pcount++;
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_IN, BEXPR> "!" {
    lastToken = BLToken.OPERATOR;
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
/* v-  *****************************************************   tu  dopisuj!   **************************************/
<PUT_RPAR, PROP_END, MPROP_END, TRANS_BEXPR_END, INIT_BEXPR_END, WHEN_END, BEXPR_MIDDLE> {BOP} {
    if(bexpnum && !bexprel)
	lastToken = BLToken.ERROR;
    else {
	lastToken = BLToken.BOPERATOR;
	yybegin(BEXPR);
	bexpbop = true;
	bexpboo = false;
	bexpnum = false;
	bexprel = false;
    }
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
/* v-  *****************************************************   tu  dopisuj!   **************************************/
<PUT_RPAR, PROP_END, MPROP_END, TRANS_BEXPR_END, INIT_BEXPR_END, WHEN_END, BEXPR_MIDDLE> {OP} {
    if(bexpboo)
	lastToken = BLToken.ERROR;
    else {
	lastToken = BLToken.OPERATOR;
	yybegin(BEXPR);
        bexpnum = true;
    }
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
/* v-  *****************************************************   tu  dopisuj!   **************************************/
<PUT_RPAR, PROP_END, MPROP_END, TRANS_BEXPR_END, INIT_BEXPR_END, WHEN_END, BEXPR_MIDDLE> {RELOP} {
    if(bexpboo)
	lastToken = BLToken.ERROR;
    else {
	lastToken = BLToken.OPERATOR;
	yybegin(BEXPR);
        bexprel = true;
    }
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<BEXPR_MIDDLE> ")" {
    if(bexp_pcount > 0)
	lastToken = BLToken.SEPARATOR_RPAREN;
    else
	lastToken = BLToken.UNMACHED_BRACKET;
    bexp_pcount--;
    String text = yytext();
    if(bexp_pcount == 0)
        yybegin(WHERE_BEXPR);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<PROP_IN, BEXPR> {Number} {
    if(bexpboo)
	lastToken = BLToken.ERROR;
    else {
	lastToken = BLToken.NUMBER;
        bexpnum = true;
	if(bexp_pcount == 0)
	    yybegin(WHERE_BEXPR);
	else
	    yybegin(BEXPR_MIDDLE);
    }
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
<PROP_IN, BEXPR> "-" {
    lastToken = BLToken.OPERATOR_SUBTRACT;
    String text = yytext();
    bexpnum = true;
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

       
    
({WhiteSpace}+) { 
    lastToken = BLToken.WHITE_SPACE;
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<YYINITIAL, CONST_DECL, VAR_DECL, BUF_DECL, PROC_DEF, LOCVAR_DECL, INIT_END, TRANS_FROM> {DocComment} {
    lastToken = BLToken.COMMENT_DOCUMENTATION;
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

{TradComment} {
    lastToken = BLToken.COMMENT_TRADITIONAL;
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
    
{OpenComment} { 
    lastToken = BLToken.ERROR_UNCLOSED_COMMENT;
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
    
{ErrorIdentifier} {
    lastToken = BLToken.ERROR_IDENTIFIER;
    String text = yytext();
    //System.out.println("error ident: "+text);
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}

<ANYTHING> {Anything} {
    lastToken = BLToken.ERROR;
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
 
 {Anything} {
    lastToken = BLToken.ERROR;
    String text = yytext();
    BLToken t = (new BLToken(lastToken,text,yyline,yychar,yychar+text.length(),zzLexicalState));
    return (t);}
